% Give predictions of data test points using the IRP path
% Strategy for each test point:
% 1. Given all points that are lower bounds, find the immediate lower bounds
% 2. Given all points that are upper bounds, find the immediate upper bounds
% 3. There are three options:
% (a) The isotonic model provides lower and upper bounds.  Interpolate: 
%     Let y_L be the maximum lower bound and y_U be the minimum upper 
%     bound.  Predict (y_L+y_U)/2.
% (b) The isotonic model provides a lower bound an no upper bounds.
%     Extrapolate: Let y_L be the maximum lower bound and predict y_L.
% (c) The isotonic model provides an upper bound an no lower bounds.
%     Extrapolate: Let y_U be the minimum upper bound and predict y_U.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:  
% pathIRP:    n x count matrix where columns are the isotonic solutions
%                 along the IRP path, and count is the length of the path
% A:           n x n matrix s.t. A(i,j)==1 if y(i)<=y(j) and A(i,j)==0 otherwise
% indsL:       indices of lower bounds, i.e. indsL{i} is a vector of the indices 
%                 of training points that are lower bounds to the ith testing point
% indsU:       indices of upper bounds, i.e. indsU{i} is a vector of the indices 
%                 of training points that are upper bounds to the ith testing point
% indStart:    index of path which which to start. e.g., if there are 30
%                 partitions in the path, indStart=5 means we want to start 
%                 returning the path after 5 partitions.
% compare:     compare(i)=1 if ith testing point is comparable to something
% incompPred:  value to predict for data points that are not comparable to
%                 any points composing the isotonic models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs:  
% yPred:       predictions of testing points along all isotonic models
%                 given by input pathIRP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function yPred = predictOutOfSample(pathIRP,A,indsL,indsU,compare,incompPred)
A=A+speye(size(A,1)); % add identity matrix to the transition matrix

numTest=length(indsL);  % number of testing points
yPred=ones(numTest,size(pathIRP,2))*incompPred;  % points that are incomparable will predict the input value incompPred

for i=1:numTest % loop through all test points, NOTE: this is slow in Matlab
    if compare(i)==1 % only run code if has comparable points
        % find all points directly below, i.e. immediate lower bounds
        indsL_direct=find(sum(A(indsL{i},indsL{i}),2)==1);
        indsL{i}=indsL{i}(indsL_direct);

        % find all points directly above, i.e. immediate upper bounds
        indsU_direct=find(sum(A(indsU{i},indsU{i}),1)==1);
        indsU{i}=indsU{i}(indsU_direct);
        
        if length(indsL{i})>0 && length(indsU{i})>0 % INTERPOLATE
            temp_path=pathIRP(indsL{i},:); % immediate lower bounds for each model along path
            [y_L,ind1]=max(temp_path,[],1); % maximum immediate lower bound
            temp_path=pathIRP(indsU{i},:); % immediate upper bounds for each model along path
            [y_U,ind2]=min(temp_path,[],1); % minimum immediate upper bound
            yPred(i,:)=(y_L+y_U)/2;
        elseif length(indsL{i})>0 % EXTRAPOLATE a point bounded only below
            temp_path=pathIRP(indsL{i},:); % immediate lower bounds for each model along path
            yPred(i,:)=max(temp_path,[],1); % maximum immediate lower bound
        elseif length(indsU{i})>0 % EXTRAPOLATE a point bounded only above
            temp_path=pathIRP(indsU{i},:); % immediate upper bounds for each model along path
            yPred(i,:)=min(temp_path,[],1); % minimum immediate lower bound
        end
    end
end